/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSHTTPCookie;

import dstep.AvailabilityMacros;
import dstep.foundation.NSArray;
import dstep.foundation.NSDate;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSHTTPCookieInternal;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSHTTPCookie_bindings;

const NSString NSHTTPCookieName;
const NSString NSHTTPCookieValue;
const NSString NSHTTPCookieOriginURL;
const NSString NSHTTPCookieVersion;
const NSString NSHTTPCookieDomain;
const NSString NSHTTPCookiePath;
const NSString NSHTTPCookieSecure;
const NSString NSHTTPCookieExpires;
const NSString NSHTTPCookieComment;
const NSString NSHTTPCookieCommentURL;
const NSString NSHTTPCookieDiscard;
const NSString NSHTTPCookieMaximumAge;
const NSString NSHTTPCookiePort;

static this ()
{
	NSHTTPCookieName = new NSString(bindings.NSHTTPCookieName);
	NSHTTPCookieValue = new NSString(bindings.NSHTTPCookieValue);
	NSHTTPCookieOriginURL = new NSString(bindings.NSHTTPCookieOriginURL);
	NSHTTPCookieVersion = new NSString(bindings.NSHTTPCookieVersion);
	NSHTTPCookieDomain = new NSString(bindings.NSHTTPCookieDomain);
	NSHTTPCookiePath = new NSString(bindings.NSHTTPCookiePath);
	NSHTTPCookieSecure = new NSString(bindings.NSHTTPCookieSecure);
	NSHTTPCookieExpires = new NSString(bindings.NSHTTPCookieExpires);
	NSHTTPCookieComment = new NSString(bindings.NSHTTPCookieComment);
	NSHTTPCookieCommentURL = new NSString(bindings.NSHTTPCookieCommentURL);
	NSHTTPCookieDiscard = new NSString(bindings.NSHTTPCookieDiscard);
	NSHTTPCookieMaximumAge = new NSString(bindings.NSHTTPCookieMaximumAge);
	NSHTTPCookiePort = new NSString(bindings.NSHTTPCookiePort);
}

class NSHTTPCookie : NSObject
{
	mixin ObjcWrap;

	Object initWithProperties (NSDictionary properties)
	{
		return invokeObjcSelf!(Object, "initWithProperties:", NSDictionary)(properties);
	}

	this (NSDictionary properties)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithProperties:", NSDictionary)(objcObject, properties);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	static Object cookieWithProperties (NSDictionary properties)
	{
		return invokeObjcSelfClass!(Object, "cookieWithProperties:", NSDictionary)(properties);
	}

	static NSDictionary requestHeaderFieldsWithCookies (NSArray cookies)
	{
		return invokeObjcSelfClass!(NSDictionary, "requestHeaderFieldsWithCookies:", NSArray)(cookies);
	}

	static NSArray cookiesWithResponseHeaderFields (NSDictionary headerFields, NSURL URL)
	{
		return invokeObjcSelfClass!(NSArray, "cookiesWithResponseHeaderFields:forURL:", NSDictionary, NSURL)(headerFields, URL);
	}

	NSDictionary properties ()
	{
		return invokeObjcSelf!(NSDictionary, "properties");
	}

	NSUInteger version_ ()
	{
		return invokeObjcSelf!(NSUInteger, "version");
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	NSString value ()
	{
		return invokeObjcSelf!(NSString, "value");
	}

	NSDate expiresDate ()
	{
		return invokeObjcSelf!(NSDate, "expiresDate");
	}

	bool isSessionOnly ()
	{
		return invokeObjcSelf!(bool, "isSessionOnly");
	}

	NSString domain ()
	{
		return invokeObjcSelf!(NSString, "domain");
	}

	NSString path ()
	{
		return invokeObjcSelf!(NSString, "path");
	}

	bool isSecure ()
	{
		return invokeObjcSelf!(bool, "isSecure");
	}

	NSString comment ()
	{
		return invokeObjcSelf!(NSString, "comment");
	}

	NSURL commentURL ()
	{
		return invokeObjcSelf!(NSURL, "commentURL");
	}

	NSArray portList ()
	{
		return invokeObjcSelf!(NSArray, "portList");
	}
}

