/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSLock;

import dstep.foundation.NSDate;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

class NSLock : NSObject, INSLocking
{
	mixin ObjcWrap;

	bool tryLock ()
	{
		return invokeObjcSelf!(bool, "tryLock");
	}

	bool lockBeforeDate (NSDate limit)
	{
		return invokeObjcSelf!(bool, "lockBeforeDate:", NSDate)(limit);
	}

	void setName (NSString n)
	{
		return invokeObjcSelf!(void, "setName:", NSString)(n);
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	void lock ()
	{
		return invokeObjcSelf!(void, "lock");
	}

	void unlock ()
	{
		return invokeObjcSelf!(void, "unlock");
	}
}

class NSCondition : NSObject, INSLocking
{
	mixin ObjcWrap;

	void wait ()
	{
		return invokeObjcSelf!(void, "wait");
	}

	bool waitUntilDate (NSDate limit)
	{
		return invokeObjcSelf!(bool, "waitUntilDate:", NSDate)(limit);
	}

	void signal ()
	{
		return invokeObjcSelf!(void, "signal");
	}

	void broadcast ()
	{
		return invokeObjcSelf!(void, "broadcast");
	}

	void setName (NSString n)
	{
		return invokeObjcSelf!(void, "setName:", NSString)(n);
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	void lock ()
	{
		return invokeObjcSelf!(void, "lock");
	}

	void unlock ()
	{
		return invokeObjcSelf!(void, "unlock");
	}
}

class NSConditionLock : NSObject, INSLocking
{
	mixin ObjcWrap;

	Object initWithCondition (NSInteger condition)
	{
		return invokeObjcSelf!(Object, "initWithCondition:", NSInteger)(condition);
	}

	this (NSInteger condition)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCondition:", NSInteger)(objcObject, condition);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	NSInteger condition ()
	{
		return invokeObjcSelf!(NSInteger, "condition");
	}

	void lockWhenCondition (NSInteger condition)
	{
		return invokeObjcSelf!(void, "lockWhenCondition:", NSInteger)(condition);
	}

	bool tryLock ()
	{
		return invokeObjcSelf!(bool, "tryLock");
	}

	bool tryLockWhenCondition (NSInteger condition)
	{
		return invokeObjcSelf!(bool, "tryLockWhenCondition:", NSInteger)(condition);
	}

	void unlockWithCondition (NSInteger condition)
	{
		return invokeObjcSelf!(void, "unlockWithCondition:", NSInteger)(condition);
	}

	bool lockBeforeDate (NSDate limit)
	{
		return invokeObjcSelf!(bool, "lockBeforeDate:", NSDate)(limit);
	}

	bool lockWhenCondition (NSInteger condition, NSDate limit)
	{
		return invokeObjcSelf!(bool, "lockWhenCondition:beforeDate:", NSInteger, NSDate)(condition, limit);
	}

	void setName (NSString n)
	{
		return invokeObjcSelf!(void, "setName:", NSString)(n);
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	void lock ()
	{
		return invokeObjcSelf!(void, "lock");
	}

	void unlock ()
	{
		return invokeObjcSelf!(void, "unlock");
	}
}

class NSRecursiveLock : NSObject, INSLocking
{
	mixin ObjcWrap;

	bool tryLock ()
	{
		return invokeObjcSelf!(bool, "tryLock");
	}

	bool lockBeforeDate (NSDate limit)
	{
		return invokeObjcSelf!(bool, "lockBeforeDate:", NSDate)(limit);
	}

	void setName (NSString n)
	{
		return invokeObjcSelf!(void, "setName:", NSString)(n);
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	void lock ()
	{
		return invokeObjcSelf!(void, "lock");
	}

	void unlock ()
	{
		return invokeObjcSelf!(void, "unlock");
	}
}

interface INSLocking
{
	void lock ();
	void unlock ();
}

