/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSMetadata;

import dstep.foundation.NSArray;
import dstep.foundation.NSDate;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSMetadataItem;
import dstep.foundation.NSMetadataQueryAttributeValueTuple;
import dstep.foundation.NSMetadataQueryResultGroup;
import dstep.foundation.NSObject;
import dstep.foundation.NSPredicate;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSMetadata_bindings;

const NSString NSMetadataQueryDidStartGatheringNotification;
const NSString NSMetadataQueryGatheringProgressNotification;
const NSString NSMetadataQueryDidFinishGatheringNotification;
const NSString NSMetadataQueryDidUpdateNotification;
const NSString NSMetadataQueryResultContentRelevanceAttribute;
const NSString NSMetadataQueryUserHomeScope;
const NSString NSMetadataQueryLocalComputerScope;
const NSString NSMetadataQueryNetworkScope;

static this ()
{
	NSMetadataQueryDidStartGatheringNotification = new NSString(bindings.NSMetadataQueryDidStartGatheringNotification);
	NSMetadataQueryGatheringProgressNotification = new NSString(bindings.NSMetadataQueryGatheringProgressNotification);
	NSMetadataQueryDidFinishGatheringNotification = new NSString(bindings.NSMetadataQueryDidFinishGatheringNotification);
	NSMetadataQueryDidUpdateNotification = new NSString(bindings.NSMetadataQueryDidUpdateNotification);
	NSMetadataQueryResultContentRelevanceAttribute = new NSString(bindings.NSMetadataQueryResultContentRelevanceAttribute);
	NSMetadataQueryUserHomeScope = new NSString(bindings.NSMetadataQueryUserHomeScope);
	NSMetadataQueryLocalComputerScope = new NSString(bindings.NSMetadataQueryLocalComputerScope);
	NSMetadataQueryNetworkScope = new NSString(bindings.NSMetadataQueryNetworkScope);
}

class NSMetadataQueryResultGroup : NSObject
{
	mixin ObjcWrap;

	NSString attribute ()
	{
		return invokeObjcSelf!(NSString, "attribute");
	}

	Object value ()
	{
		return invokeObjcSelf!(Object, "value");
	}

	NSArray subgroups ()
	{
		return invokeObjcSelf!(NSArray, "subgroups");
	}

	NSUInteger resultCount ()
	{
		return invokeObjcSelf!(NSUInteger, "resultCount");
	}

	Object resultAtIndex (NSUInteger idx)
	{
		return invokeObjcSelf!(Object, "resultAtIndex:", NSUInteger)(idx);
	}

	NSArray results ()
	{
		return invokeObjcSelf!(NSArray, "results");
	}
}

class NSMetadataItem : NSObject
{
	mixin ObjcWrap;

	Object valueForAttribute (NSString key)
	{
		return invokeObjcSelf!(Object, "valueForAttribute:", NSString)(key);
	}

	NSDictionary valuesForAttributes (NSArray keys)
	{
		return invokeObjcSelf!(NSDictionary, "valuesForAttributes:", NSArray)(keys);
	}

	NSArray attributes ()
	{
		return invokeObjcSelf!(NSArray, "attributes");
	}
}

class NSMetadataQuery : NSObject
{
	mixin ObjcWrap;

	Object init ()
	{
		return invokeObjcSelf!(Object, "init");
	}

	this ()
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "init")(objcObject);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}

	NSPredicate predicate ()
	{
		return invokeObjcSelf!(NSPredicate, "predicate");
	}

	void setPredicate (NSPredicate predicate)
	{
		return invokeObjcSelf!(void, "setPredicate:", NSPredicate)(predicate);
	}

	NSArray sortDescriptors ()
	{
		return invokeObjcSelf!(NSArray, "sortDescriptors");
	}

	void setSortDescriptors (NSArray descriptors)
	{
		return invokeObjcSelf!(void, "setSortDescriptors:", NSArray)(descriptors);
	}

	NSArray valueListAttributes ()
	{
		return invokeObjcSelf!(NSArray, "valueListAttributes");
	}

	void setValueListAttributes (NSArray attrs)
	{
		return invokeObjcSelf!(void, "setValueListAttributes:", NSArray)(attrs);
	}

	NSArray groupingAttributes ()
	{
		return invokeObjcSelf!(NSArray, "groupingAttributes");
	}

	void setGroupingAttributes (NSArray attrs)
	{
		return invokeObjcSelf!(void, "setGroupingAttributes:", NSArray)(attrs);
	}

	double notificationBatchingInterval ()
	{
		return invokeObjcSelf!(double, "notificationBatchingInterval");
	}

	void setNotificationBatchingInterval (double ti)
	{
		return invokeObjcSelf!(void, "setNotificationBatchingInterval:", double)(ti);
	}

	NSArray searchScopes ()
	{
		return invokeObjcSelf!(NSArray, "searchScopes");
	}

	void setSearchScopes (NSArray scopes)
	{
		return invokeObjcSelf!(void, "setSearchScopes:", NSArray)(scopes);
	}

	bool startQuery ()
	{
		return invokeObjcSelf!(bool, "startQuery");
	}

	void stopQuery ()
	{
		return invokeObjcSelf!(void, "stopQuery");
	}

	bool isStarted ()
	{
		return invokeObjcSelf!(bool, "isStarted");
	}

	bool isGathering ()
	{
		return invokeObjcSelf!(bool, "isGathering");
	}

	bool isStopped ()
	{
		return invokeObjcSelf!(bool, "isStopped");
	}

	void disableUpdates ()
	{
		return invokeObjcSelf!(void, "disableUpdates");
	}

	void enableUpdates ()
	{
		return invokeObjcSelf!(void, "enableUpdates");
	}

	NSUInteger resultCount ()
	{
		return invokeObjcSelf!(NSUInteger, "resultCount");
	}

	Object resultAtIndex (NSUInteger idx)
	{
		return invokeObjcSelf!(Object, "resultAtIndex:", NSUInteger)(idx);
	}

	NSArray results ()
	{
		return invokeObjcSelf!(NSArray, "results");
	}

	NSUInteger indexOfResult (Object result)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfResult:", Object)(result);
	}

	NSDictionary valueLists ()
	{
		return invokeObjcSelf!(NSDictionary, "valueLists");
	}

	NSArray groupedResults ()
	{
		return invokeObjcSelf!(NSArray, "groupedResults");
	}

	Object valueOfAttribute (NSString attrName, NSUInteger idx)
	{
		return invokeObjcSelf!(Object, "valueOfAttribute:forResultAtIndex:", NSString, NSUInteger)(attrName, idx);
	}
}

class NSMetadataQueryAttributeValueTuple : NSObject
{
	mixin ObjcWrap;

	NSString attribute ()
	{
		return invokeObjcSelf!(NSString, "attribute");
	}

	Object value ()
	{
		return invokeObjcSelf!(Object, "value");
	}

	NSUInteger count ()
	{
		return invokeObjcSelf!(NSUInteger, "count");
	}
}

template TNSMetadataQueryDelegate ()
{
	Object metadataQuery (NSMetadataQuery query, NSMetadataItem result);
	Object metadataQuery (NSMetadataQuery query, NSString attrName, Object attrValue);
}

