/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSPortCoder;

import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSConnection;
import dstep.foundation.NSPort;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

class NSPortCoder : NSCoder
{
	mixin ObjcWrap;

	bool isBycopy ()
	{
		return invokeObjcSelf!(bool, "isBycopy");
	}

	bool isByref ()
	{
		return invokeObjcSelf!(bool, "isByref");
	}

	NSConnection connection ()
	{
		return invokeObjcSelf!(NSConnection, "connection");
	}

	void encodePortObject (NSPort aport)
	{
		return invokeObjcSelf!(void, "encodePortObject:", NSPort)(aport);
	}

	NSPort decodePortObject ()
	{
		return invokeObjcSelf!(NSPort, "decodePortObject");
	}

	static Object portCoderWithReceivePort (NSPort rcvPort, NSPort sndPort, NSArray comps)
	{
		return invokeObjcSelfClass!(Object, "portCoderWithReceivePort:sendPort:components:", NSPort, NSPort, NSArray)(rcvPort, sndPort, comps);
	}

	Object initWithReceivePort (NSPort rcvPort, NSPort sndPort, NSArray comps)
	{
		return invokeObjcSelf!(Object, "initWithReceivePort:sendPort:components:", NSPort, NSPort, NSArray)(rcvPort, sndPort, comps);
	}

	this (NSPort rcvPort, NSPort sndPort, NSArray comps)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithReceivePort:sendPort:components:", NSPort, NSPort, NSArray)(objcObject, rcvPort, sndPort, comps);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void dispatch ()
	{
		return invokeObjcSelf!(void, "dispatch");
	}
}

template TNSDistributedObjects ()
{
	Class classForPortCoder ();
	Object replacementObjectForPortCoder (NSPortCoder coder);
}

