/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSXMLDTD;

import dstep.foundation.NSData;
import dstep.foundation.NSMutableDictionary;
import dstep.foundation.NSXMLDTDNode;
import dstep.foundation.NSXMLNode;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

class NSXMLDTD : NSXMLNode
{
	mixin ObjcWrap;

	Object initWithContentsOfURL (NSURL url, NSUInteger mask, NSError** error)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfURL:options:error:", NSURL, NSUInteger, NSError**)(url, mask, error);
	}

	this (NSURL url, NSUInteger mask, NSError** error)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithContentsOfURL:options:error:", NSURL, NSUInteger, NSError**)(objcObject, url, mask, error);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithData (NSData data, NSUInteger mask, NSError** error)
	{
		return invokeObjcSelf!(Object, "initWithData:options:error:", NSData, NSUInteger, NSError**)(data, mask, error);
	}

	this (NSData data, NSUInteger mask, NSError** error)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithData:options:error:", NSData, NSUInteger, NSError**)(objcObject, data, mask, error);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void setPublicID (NSString publicID)
	{
		return invokeObjcSelf!(void, "setPublicID:", NSString)(publicID);
	}

	NSString publicID ()
	{
		return invokeObjcSelf!(NSString, "publicID");
	}

	void setSystemID (NSString systemID)
	{
		return invokeObjcSelf!(void, "setSystemID:", NSString)(systemID);
	}

	NSString systemID ()
	{
		return invokeObjcSelf!(NSString, "systemID");
	}

	void insertChild (NSXMLNode child, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChild:atIndex:", NSXMLNode, NSUInteger)(child, index);
	}

	void insertChildren (NSArray children, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChildren:atIndex:", NSArray, NSUInteger)(children, index);
	}

	void removeChildAtIndex (NSUInteger index)
	{
		return invokeObjcSelf!(void, "removeChildAtIndex:", NSUInteger)(index);
	}

	void setChildren (NSArray children)
	{
		return invokeObjcSelf!(void, "setChildren:", NSArray)(children);
	}

	void addChild (NSXMLNode child)
	{
		return invokeObjcSelf!(void, "addChild:", NSXMLNode)(child);
	}

	void replaceChildAtIndex (NSUInteger index, NSXMLNode node)
	{
		return invokeObjcSelf!(void, "replaceChildAtIndex:withNode:", NSUInteger, NSXMLNode)(index, node);
	}

	NSXMLDTDNode entityDeclarationForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLDTDNode, "entityDeclarationForName:", NSString)(name);
	}

	NSXMLDTDNode notationDeclarationForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLDTDNode, "notationDeclarationForName:", NSString)(name);
	}

	NSXMLDTDNode elementDeclarationForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLDTDNode, "elementDeclarationForName:", NSString)(name);
	}

	NSXMLDTDNode attributeDeclarationForName (NSString name, NSString elementName)
	{
		return invokeObjcSelf!(NSXMLDTDNode, "attributeDeclarationForName:elementName:", NSString, NSString)(name, elementName);
	}

	static NSXMLDTDNode predefinedEntityDeclarationForName (NSString name)
	{
		return invokeObjcSelfClass!(NSXMLDTDNode, "predefinedEntityDeclarationForName:", NSString)(name);
	}
}

